@@ -267,7 +267,13 @@ public class RelayNetworkStatusVoteImplTest {
       vb.dirKeyCertificationLines = lines;
       return new RelayNetworkStatusVoteImpl(vb.buildVote(), true);
     }
-    private List<String> statusEntries = new ArrayList<String>();
+    private List<String> statusEntries = null;
+    private static RelayNetworkStatusVote createWithStatusEntries(
+        List<String> statusEntries) throws DescriptorParseException {
+      VoteBuilder vb = new VoteBuilder();
+      vb.statusEntries = statusEntries;
+      return new RelayNetworkStatusVoteImpl(vb.buildVote(), true);
+    }
     private String directoryFooterLine = "directory-footer";
     private static RelayNetworkStatusVote
         createWithDirectoryFooterLine(String line)
@@ -343,6 +349,10 @@ public class RelayNetworkStatusVoteImplTest {
     }
 
     private VoteBuilder() {
+      if (this.statusEntries != null) {
+        return;
+      }
+      this.statusEntries = new ArrayList<>();
       this.statusEntries.add("r right2privassy3 "
           + "ADQ6gCT3DiFHKPDFr3rODBUI8HM lJY5Vf7kXec+VdkGW2flEsfkFC8 "
           + "2011-11-12 00:03:40 50.63.8.215 9023 0\n"
@@ -368,108 +378,108 @@ public class RelayNetworkStatusVoteImplTest {
     }
     private void appendHeader(StringBuilder sb) {
       if (this.networkStatusVersionLine != null) {
-        sb.append(this.networkStatusVersionLine + "\n");
+        sb.append(this.networkStatusVersionLine).append("\n");
       }
       if (this.voteStatusLine != null) {
-        sb.append(this.voteStatusLine + "\n");
+        sb.append(this.voteStatusLine).append("\n");
       }
       if (this.consensusMethodsLine != null) {
-        sb.append(this.consensusMethodsLine + "\n");
+        sb.append(this.consensusMethodsLine).append("\n");
       }
       if (this.publishedLine != null) {
-        sb.append(this.publishedLine + "\n");
+        sb.append(this.publishedLine).append("\n");
       }
       if (this.validAfterLine != null) {
-        sb.append(this.validAfterLine + "\n");
+        sb.append(this.validAfterLine).append("\n");
       }
       if (this.freshUntilLine != null) {
-        sb.append(this.freshUntilLine + "\n");
+        sb.append(this.freshUntilLine).append("\n");
       }
       if (this.validUntilLine != null) {
-        sb.append(this.validUntilLine + "\n");
+        sb.append(this.validUntilLine).append("\n");
       }
       if (this.votingDelayLine != null) {
-        sb.append(this.votingDelayLine + "\n");
+        sb.append(this.votingDelayLine).append("\n");
       }
       if (this.clientVersionsLine != null) {
-        sb.append(this.clientVersionsLine + "\n");
+        sb.append(this.clientVersionsLine).append("\n");
       }
       if (this.serverVersionsLine != null) {
-        sb.append(this.serverVersionsLine + "\n");
+        sb.append(this.serverVersionsLine).append("\n");
       }
       if (this.knownFlagsLine != null) {
-        sb.append(this.knownFlagsLine + "\n");
+        sb.append(this.knownFlagsLine).append("\n");
       }
       if (this.flagThresholdsLine != null) {
-        sb.append(this.flagThresholdsLine + "\n");
+        sb.append(this.flagThresholdsLine).append("\n");
       }
       if (this.paramsLine != null) {
-        sb.append(this.paramsLine + "\n");
+        sb.append(this.paramsLine).append("\n");
       }
       if (this.unrecognizedHeaderLine != null) {
-        sb.append(this.unrecognizedHeaderLine + "\n");
+        sb.append(this.unrecognizedHeaderLine).append("\n");
       }
     }
     private void appendDirSource(StringBuilder sb) {
       if (this.dirSourceLine != null) {
-        sb.append(this.dirSourceLine + "\n");
+        sb.append(this.dirSourceLine).append("\n");
       }
       if (this.contactLine != null) {
-        sb.append(this.contactLine + "\n");
+        sb.append(this.contactLine).append("\n");
       }
       if (this.legacyDirKeyLine != null) {
-        sb.append(this.legacyDirKeyLine + "\n");
+        sb.append(this.legacyDirKeyLine).append("\n");
       }
       if (this.dirKeyCertificateVersionLine != null) {
-        sb.append(this.dirKeyCertificateVersionLine + "\n");
+        sb.append(this.dirKeyCertificateVersionLine).append("\n");
       }
       if (this.fingerprintLine != null) {
-        sb.append(this.fingerprintLine + "\n");
+        sb.append(this.fingerprintLine).append("\n");
       }
       if (this.dirKeyPublishedLine != null) {
-        sb.append(this.dirKeyPublishedLine + "\n");
+        sb.append(this.dirKeyPublishedLine).append("\n");
       }
       if (this.dirKeyExpiresLine != null) {
-        sb.append(this.dirKeyExpiresLine + "\n");
+        sb.append(this.dirKeyExpiresLine).append("\n");
       }
       if (this.dirIdentityKeyLines != null) {
-        sb.append(this.dirIdentityKeyLines + "\n");
+        sb.append(this.dirIdentityKeyLines).append("\n");
       }
       if (this.dirSigningKeyLines != null) {
-        sb.append(this.dirSigningKeyLines + "\n");
+        sb.append(this.dirSigningKeyLines).append("\n");
       }
       if (this.dirKeyCrosscertLines != null) {
-        sb.append(this.dirKeyCrosscertLines + "\n");
+        sb.append(this.dirKeyCrosscertLines).append("\n");
       }
       if (this.dirKeyCertificationLines != null) {
-        sb.append(this.dirKeyCertificationLines + "\n");
+        sb.append(this.dirKeyCertificationLines).append("\n");
       }
       if (this.unrecognizedDirSourceLine != null) {
-        sb.append(this.unrecognizedDirSourceLine + "\n");
+        sb.append(this.unrecognizedDirSourceLine).append("\n");
       }
     }
     private void appendStatusEntries(StringBuilder sb) {
       for (String statusEntry : this.statusEntries) {
-        sb.append(statusEntry + "\n");
+        sb.append(statusEntry).append("\n");
       }
       if (this.unrecognizedStatusEntryLine != null) {
-        sb.append(this.unrecognizedStatusEntryLine + "\n");
+        sb.append(this.unrecognizedStatusEntryLine).append("\n");
       }
     }
     private void appendFooter(StringBuilder sb) {
       if (this.directoryFooterLine != null) {
-        sb.append(this.directoryFooterLine + "\n");
+        sb.append(this.directoryFooterLine).append("\n");
       }
       if (this.unrecognizedFooterLine != null) {
-        sb.append(this.unrecognizedFooterLine + "\n");
+        sb.append(this.unrecognizedFooterLine).append("\n");
       }
     }
     private void appendDirectorySignature(StringBuilder sb) {
       if (this.directorySignatureLines != null) {
-        sb.append(directorySignatureLines + "\n");
+        sb.append(directorySignatureLines).append("\n");
       }
       if (this.unrecognizedDirectorySignatureLine != null) {
-        sb.append(this.unrecognizedDirectorySignatureLine + "\n");
+        sb.append(this.unrecognizedDirectorySignatureLine).append("\n");
       }
     }
   }
@@ -1140,7 +1150,7 @@ public class RelayNetworkStatusVoteImplTest {
     String unrecognizedLine = "unrecognized-line 1";
     RelayNetworkStatusVote vote = VoteBuilder.
         createWithUnrecognizedHeaderLine(unrecognizedLine, false);
-    List<String> unrecognizedLines = new ArrayList<String>();
+    List<String> unrecognizedLines = new ArrayList<>();
     unrecognizedLines.add(unrecognizedLine);
     assertEquals(unrecognizedLines, vote.getUnrecognizedLines());
   }
@@ -1159,7 +1169,7 @@ public class RelayNetworkStatusVoteImplTest {
     String unrecognizedLine = "unrecognized-line 1";
     RelayNetworkStatusVote vote = VoteBuilder.
         createWithUnrecognizedDirSourceLine(unrecognizedLine, false);
-    List<String> unrecognizedLines = new ArrayList<String>();
+    List<String> unrecognizedLines = new ArrayList<>();
     unrecognizedLines.add(unrecognizedLine);
     assertEquals(unrecognizedLines, vote.getUnrecognizedLines());
   }
@@ -1177,9 +1187,51 @@ public class RelayNetworkStatusVoteImplTest {
     String unrecognizedLine = "unrecognized-line 1";
     RelayNetworkStatusVote vote = VoteBuilder.
         createWithUnrecognizedFooterLine(unrecognizedLine, false);
-    List<String> unrecognizedLines = new ArrayList<String>();
+    List<String> unrecognizedLines = new ArrayList<>();
     unrecognizedLines.add(unrecognizedLine);
     assertEquals(unrecognizedLines, vote.getUnrecognizedLines());
   }
+
+  @Test()
+  public void testIdEd25519MasterKey()
+      throws DescriptorParseException {
+    String masterKey25519 = "8RH34kO07Pp+XYwzdoATVyCibIvmbslUjRkAm7J4IA8";
+    List<String> statusEntries = new ArrayList<>();
+    statusEntries.add("r PDrelay1 AAFJ5u9xAqrKlpDW6N0pMhJLlKs "
+        + "bgJiI/la3e9u0K7cQ5pMSXhigHI 2015-12-01 04:54:30 95.215.44.189 "
+        + "8080 0\n"
+        + "id ed25519 " + masterKey25519);
+    RelayNetworkStatusVote vote =
+        VoteBuilder.createWithStatusEntries(statusEntries);
+    String fingerprint = vote.getStatusEntries().firstKey();
+    assertEquals(masterKey25519,
+        vote.getStatusEntry(fingerprint).getMasterKeyEd25519());
+  }
+
+  @Test()
+  public void testIdEd25519None()
+      throws DescriptorParseException {
+    List<String> statusEntries = new ArrayList<>();
+    statusEntries.add("r MathematicalApology AAPJIrV9nhfgTYQs0vsTghFaP2A "
+        + "uA7p0m68O8ILXsf3aLZUj0EvDNE 2015-12-01 18:01:49 172.99.69.177 "
+        + "443 9030\n"
+        + "id ed25519 none");
+    RelayNetworkStatusVote vote =
+        VoteBuilder.createWithStatusEntries(statusEntries);
+    String fingerprint = vote.getStatusEntries().firstKey();
+    assertEquals("none",
+        vote.getStatusEntry(fingerprint).getMasterKeyEd25519());
+  }
+
+  @Test(expected = DescriptorParseException.class)
+  public void testIdRsa1024None()
+      throws DescriptorParseException {
+    List<String> statusEntries = new ArrayList<>();
+    statusEntries.add("r MathematicalApology AAPJIrV9nhfgTYQs0vsTghFaP2A "
+        + "uA7p0m68O8ILXsf3aLZUj0EvDNE 2015-12-01 18:01:49 172.99.69.177 "
+        + "443 9030\n"
+        + "id rsa1024 none");
+    VoteBuilder.createWithStatusEntries(statusEntries);
+  }
 }
 
